#!/usr/bin/perl

## --> 3D Local Ground Elevation Map, 3dmap.cgi
## --> Green Bay Professional Packet Radio, www.gbppr.org

## This file Copyright 2006 <contact@gbppr.org> under the GPL.
## NO WARRANTY.  Please send bug reports / patches.

# Setup
#
$ENV{PATH} = "/bin:/usr/bin:/usr/local/bin";
select STDOUT;
$| = 1;
use Math::Trig;
use Math::Complex;

require "flush.pl";

# Print MIME
#
print "Content-type:text/html\n\n";
&flush(STDOUT);

# Read environment
#
read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
@pairs = split(/&/, $buffer);
foreach $pair (@pairs) {
  ($name, $value) = split(/=/, $pair);
  $value =~ tr/+/ /;
  $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
  $FORM{$name} = $value;
}

my $LAT1_D = $FORM{'LAT1_D'};
my $LAT1_M = $FORM{'LAT1_M'};
my $LAT1_S = $FORM{'LAT1_S'};

my $LON1_D = $FORM{'LON1_D'};
my $LON1_M = $FORM{'LON1_M'};
my $LON1_S = $FORM{'LON1_S'};

# Begin
#
($sec,$min,$hour,$mday,$mon,$year) = gmtime;
$hour = sprintf "%02.0f",$hour;
$min  = sprintf "%02.0f",$min;
$sec  = sprintf "%02.0f",$sec;
$mon  = sprintf "%02.0f",($mon+1);
$mday = sprintf "%02.0f",$mday;
$year = sprintf "%02.0f",$year;
$year = 1900 + $year;

$RAN = sprintf "%.f", rand(10000000);
mkdir "tmp/$mon-$mday";
mkdir "tmp/$mon-$mday/$RAN";
chdir "tmp/$mon-$mday/$RAN";
unlink <*>;
symlink "/usr/splat/sdf", "sdf";

sub System {
if ((0xffff & system $args) != 0 ) {
  print STDERR "error: $!\n";
  exit 1;
 }
}

$LAT1_D =~ tr/0-9.//csd;
$LAT1_M =~ tr/0-9.//csd;
$LAT1_S =~ tr/0-9.//csd;

$LON1_D =~ tr/0-9.//csd;
$LON1_M =~ tr/0-9.//csd;
$LON1_S =~ tr/0-9.//csd;

if ($LAT1_D > 72 || $LAT1_D < 17) {
  print "<b>BAD COORDINATES! - $LAT1_D &gt; 72 or $LAT1_D &lt; 17</b>\n";
  exit 1;
}
if ($LAT1_M > 60) {
  print "<b>BAD COORDINATES! - $LAT1_M &gt; 60</b>\n";
  exit 1;
}
if ($LAT1_S > 60) {
  print "<b>BAD COORDINATES! - $LAT1_S &gt; 60</b>\n"; 
  exit 1;
}

if ($LON1_D > 180) {
  print "<b>BAD COORDINATES! - $LON1_D &gt; 180</b>\n";
  exit 1;
}
if ($LON1_M > 60) {
  print "<b>BAD COORDINATES! - $LON1_M &gt; 60</b>\n"; 
  exit 1;
}
if ($LON1_S > 60) {
  print "<b>BAD COORDINATES! - $LON1_S &gt; 60</b>\n"; 
  exit 1;
}

$LAT1 = sprintf "%.6f", $LAT1_D + ($LAT1_M / 60) + ($LAT1_S / 3600);
$LON1 = sprintf "%.6f", $LON1_D + ($LON1_M / 60) + ($LON1_S / 3600);

$LAT1_D = sprintf "%02.0f", $LAT1_D;
$LAT1_M = sprintf "%02.0f", $LAT1_M;
$LAT1_S = sprintf "%02.2f", $LAT1_S;

$LON1_D = sprintf "%03.0f", $LON1_D;
$LON1_M = sprintf "%02.0f", $LON1_M;
$LON1_S = sprintf "%02.2f", $LON1_S;


# Draw me a web page
#
print <<EOF;
<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 3.2//EN">
<html>
<head>
<title>3D Local Ground Elevation Map Results</title>
</head>
<body bgcolor="#D3D3D3" text="#000000" link="blue">
<center>
<h2>3D Local Ground Elevation Map Results</h2>
</center>

<hr noshade>
<pre>
<u><b>Transmitter Site</b></u>

                          <b>Latitude : </b>$LAT1<b>&deg; (</b>$LAT1_D<b>&deg; </b>$LAT1_M<b>' </b>$LAT1_S<b>") North</b>
                         <b>Longitude : </b>$LON1<b>&deg; (</b>$LON1_D<b>&deg; </b>$LON1_M<b>' </b>$LON1_S<b>") West</b>
EOF
&flush(STDOUT);


$LAT_START = sprintf "%.6f", $LAT1 - 0.006;
$LAT_STOP = sprintf "%.6f", $LAT1 + 0.006;

$LON_START = sprintf "%.6f", $LON1 - 0.006;
$LON_STOP = sprintf "%.6f", $LON1 + 0.006;

print "\n\n<p><b>Starting ... will take a few minutes/hours/years.  (Click image for PDF file)</b></p>\n";
&flush(STDOUT);

open D, "> 3D.data";
while ($LON_START < $LON_STOP) {

    while ($LAT_START < $LAT_STOP) {
      open F, "> tx.qth";
        printf F "null\n%.6f\n%.6f\n0\n", $LAT_START, $LON_START;
      close F;

      open S, "/usr/local/bin/splat.elv -t tx.qth -r tx.qth -d sdf |";
	while (<S>) {
	  chomp;
	  $elev_m = $_ * 0.3048;
	}
      close S;

      printf D "%.6f\t%.6f\t%.f\n", $LON_START, $LAT_START, $elev_m;
      $LAT_START = sprintf "%.6f", $LAT_START + 0.002;
    }

    $LON_START = sprintf "%.6f", $LON_START + 0.002;
    $LAT_START = sprintf "%.6f", $LAT1 - 0.006;
    $LAT_STOP = sprintf "%.6f", $LAT1 + 0.006;
    print D "\n";
}
close D;

open A, "> 3D1.gp";
  print A "unset ztics\n".
	  "set title \"GBPPR 3D Ground Elevation (meters)\"\n".
          "set grid\n".
          "set noclabel\n".
          "set pm3d\n".
          "set cntrparam bspline\n".
          "set contour base\n".
          "set cntrparam levels 10\n".
          "set surface\n".
          "set nokey\n".
          "set xlabel \"Latitude\\n(Degrees North)\"\n".
          "set ylabel \"Longitude\\n(Degrees West)\"\n".
	  "set xrange [] reverse\n".
          "set auto\n".
          "set samples 300\n".
          "set xtics nomirror\n".
          "set ytics nomirror\n".
          "set tics out\n".
          "set ticslevel 0\n".
          "set hidden3d\n".
          "set dgrid3d 40,51,4\n".
          "set view 60,30,1\n".
          "set border 15 lw 1\n".
          "set ticscale 1.5\n".
          "set output \"GBPPR_3D_Map1.ps\"\n".
          "set terminal postscript enhanced color\n".
          "splot \"3D.data\" using 2:1:3 with lines lt 2 lw 2\n".
	  "set output \"GBPPR_3D_Map1.png\"\n".
	  "set terminal png\n".
	  "replot\n".
	  "set title \"GBPPR 3D Ground Elevation (meters) - Alternate View\"\n".
          "set view 20,60,1\n".
	  "set output \"GBPPR_3D_Map3.ps\"\n".
	  "set terminal postscript enhanced color\n".
	  "replot\n".
	  "set output \"GBPPR_3D_Map3.png\"\n".
          "set terminal png\n".
	  "replot\n";
close A;

&System($args = "/usr/local/bin/gnuplot 3D1.gp");
&System($args = "/usr/bin/ps2pdf14 GBPPR_3D_Map1.ps GBPPR_3D_Map1.pdf");
&System($args = "/usr/bin/ps2pdf14 GBPPR_3D_Map3.ps GBPPR_3D_Map3.pdf");

open C, "> 3D2.gp";
  print C "unset ztics\n".
          "set title \"GBPPR 3D Ground Elevation (meters)\"\n".
	  "set noclabel\n".
	  "set pm3d map\n".
          "set nokey\n".
          "set ylabel \"Latitude\\n(Degrees North)\"\n".
          "set xlabel \"Longitude\\n(Degrees West)\"\n".
          "set auto\n".
          "set samples 300\n".
          "set xtics nomirror\n".
          "set ytics nomirror\n".
          "set tics out\n".
          "set hidden3d\n".
          "set dgrid3d 40,51,4\n".
          "set ticscale 1.5\n".
	  "set output \"GBPPR_3D_Map2.ps\"\n".
	  "set terminal postscript enhanced color\n".
	  "splot \"3D.data\" with lines lt 2\n".
          "set output \"GBPPR_3D_Map2.png\"\n".
          "set terminal png\n".
          "replot\n".
	  "set cntrparam bspline\n".
	  "set contour base\n".
	  "set cntrparam levels 10\n".
          "set output \"GBPPR_3D_Map4.ps\"\n".
	  "set terminal postscript enhanced color\n".
	  "replot\n".
          "set output \"GBPPR_3D_Map4.png\"\n".
	  "set terminal png\n".
          "replot\n";
close C;

&System($args = "/usr/local/bin/gnuplot 3D2.gp");
#&System($args = "/usr/bin/ps2pdf14 GBPPR_3D_Map2.ps GBPPR_3D_Map2.pdf");
&System($args = "/usr/bin/ps2pdf14 GBPPR_3D_Map4.ps GBPPR_3D_Map4.pdf");

print "<center><p><a href=\"tmp/$mon-$mday/$RAN/GBPPR_3D_Map1.pdf\"><img src=\"tmp/$mon-$mday/$RAN/GBPPR_3D_Map1.png\"></a></p></center>\n";
print "<center><p><a href=\"tmp/$mon-$mday/$RAN/GBPPR_3D_Map3.pdf\"><img src=\"tmp/$mon-$mday/$RAN/GBPPR_3D_Map3.png\"></a></p></center>\n";

#print "<center><p><a href=\"tmp/$mon-$mday/$RAN/GBPPR_3D_Map2.pdf\"><img src=\"tmp/$mon-$mday/$RAN/GBPPR_3D_Map2.png\"></a></p></center>\n";
print "<center><p><a href=\"tmp/$mon-$mday/$RAN/GBPPR_3D_Map4.pdf\"><img src=\"tmp/$mon-$mday/$RAN/GBPPR_3D_Map4.png\"></a></p></center>\n";

print "<p>Calculated on $mon/$mday/$year</p>\n";
print "</body>\n";
print "</html>\n";
&flush(STDOUT);

unlink "tx.qth", "sdf", "3D.data", "3D1.gp", "3D2.gp", "GBPPR_3D_Map1.ps", "GBPPR_3D_Map2.ps", "GBPPR_3D_Map3.ps", "GBPPR_3D_Map4.ps";

exit 0;
