#!/usr/bin/perl

## --> Longley-Rice Path Loss Analysis, longley.cgi
## --> Green Bay Professional Packet Radio, www.gbppr.org

## This file Copyright 2004 <contact@gbppr.org> under the GPL.
## NO WARRANTY.  Please send bug reports / patches.

# Setup
#
$ENV{PATH} = "/bin:/usr/bin:/usr/local/bin";
select STDOUT;
$| = 1;
use Math::Trig;
use Math::Complex;

require "flush.pl";

# Print MIME
#
print "Content-type:text/html\n\n";
&flush(STDOUT);

# Read environment
#
read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
@pairs = split(/&/, $buffer);
foreach $pair (@pairs) {
  ($name, $value) = split(/=/, $pair);
  $value =~ tr/+/ /;
  $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
  $FORM{$name} = $value;
}

my $LAT1_D = $FORM{'LAT1_D'};
my $LAT1_M = $FORM{'LAT1_M'};
my $LAT1_S = $FORM{'LAT1_S'};

my $LON1_D = $FORM{'LON1_D'};
my $LON1_M = $FORM{'LON1_M'};
my $LON1_S = $FORM{'LON1_S'};

my $tx_ant_ht_val = $FORM{'tx_ant_ht_val'};
my $tx_ant_ht = $FORM{'tx_ant_ht'};

my $frq = $FORM{'frq'};
my $frq_val = $FORM{'frq_val'};

my $polar = $FORM{'polar'};

my $coverage = $FORM{'coverage'};
my $coverage_val = $FORM{'coverage_val'};

my $maxdb = $FORM{'maxdb'};

my $state = $FORM{'state'};

my $rx_ant_ht_val = $FORM{'rx_ant_ht_val'};
my $rx_ant_ht = $FORM{'rx_ant_ht'};

my $diecon = $FORM{'diecon'};
my $earcon = $FORM{'earcon'};
my $climate = $FORM{'climate'};
my $sit = $FORM{'sit'};
my $time = $FORM{'time'};
my $k = $FORM{'k'};

my $quality = $FORM{'quality'};

# Begin
#
($sec,$min,$hour,$mday,$mon,$year) = gmtime;
$hour = sprintf "%02.0f",$hour;
$min  = sprintf "%02.0f",$min;
$sec  = sprintf "%02.0f",$sec;
$mon  = sprintf "%02.0f",($mon+1);
$mday = sprintf "%02.0f",$mday;
$year = sprintf "%02.0f",$year;
$year = 1900 + $year;

$RAN = sprintf "%.f", rand(10000000);
mkdir "tmp/$mon-$mday";
mkdir "tmp/$mon-$mday/$RAN";
chdir "tmp/$mon-$mday/$RAN";
unlink <*>;
symlink "/usr/splat/sdf", "sdf";

sub System {
if ((0xffff & system $args) != 0 ) {
  print STDERR "error: $!\n";
  exit 1;
 }
}

$LAT1_D =~ tr/0-9.//csd;
$LAT1_M =~ tr/0-9.//csd;
$LAT1_S =~ tr/0-9.//csd;

$LON1_D =~ tr/0-9.//csd;
$LON1_M =~ tr/0-9.//csd;
$LON1_S =~ tr/0-9.//csd;

if ($LAT1_D > 72 || $LAT1_D < 17) {
  print "<b>BAD COORDINATES! - $LAT1_D &gt; 72 or $LAT1_D &lt; 17</b>\n";
  exit 1;
}
if ($LAT1_M > 60) {
  print "<b>BAD COORDINATES! - $LAT1_M &gt; 60</b>\n";
  exit 1;
}
if ($LAT1_S > 60) {
  print "<b>BAD COORDINATES! - $LAT1_S &gt; 60</b>\n"; 
  exit 1;
}

if ($LON1_D > 180) {
  print "<b>BAD COORDINATES! - $LON1_D &gt; 180</b>\n";
  exit 1;
}
if ($LON1_M > 60) {
  print "<b>BAD COORDINATES! - $LON1_M &gt; 60</b>\n"; 
  exit 1;
}
if ($LON1_S > 60) {
  print "<b>BAD COORDINATES! - $LON1_S &gt; 60</b>\n"; 
  exit 1;
}

$LAT1 = sprintf "%.6f", $LAT1_D + ($LAT1_M / 60) + ($LAT1_S / 3600);
$LON1 = sprintf "%.6f", $LON1_D + ($LON1_M / 60) + ($LON1_S / 3600);

$LAT1_D = sprintf "%02.0f", $LAT1_D;
$LAT1_M = sprintf "%02.0f", $LAT1_M;
$LAT1_S = sprintf "%02.2f", $LAT1_S;

$LON1_D = sprintf "%03.0f", $LON1_D;
$LON1_M = sprintf "%02.0f", $LON1_M;
$LON1_S = sprintf "%02.2f", $LON1_S;

$rx_ant_ht =~ tr/0-9.//csd;

$tx_ant_ht =~ tr/0-9.//csd;

if (!$rx_ant_ht) {
  $rx_ant_ht = 1.5;
  $rx_ant_ht_val = "meters"; 
}

if (!$tx_ant_ht) {
  $tx_ant_ht = 8;
  $tx_ant_ht_val = "meters";
}

if ($rx_ant_ht_val eq "feet") {
  $rx_ant_ht_m = sprintf "%.1f", $rx_ant_ht * 0.3048; # feet to meters
  $rx_ant_ht_ft = sprintf "%.1f", $rx_ant_ht; 
} 
else {
  $rx_ant_ht_m = sprintf "%.1f", $rx_ant_ht;
  $rx_ant_ht_ft = sprintf "%.1f", $rx_ant_ht / 0.3048;
} 

if ($tx_ant_ht_val eq "feet") {
  $tx_ant_ht_m = sprintf "%.1f", $tx_ant_ht * 0.3048; # feet to meters
  $tx_ant_ht_ft = sprintf "%.1f", $tx_ant_ht;
}
else {
  $tx_ant_ht_m = sprintf "%.1f", $tx_ant_ht;
  $tx_ant_ht_ft = sprintf "%.1f", $tx_ant_ht / 0.3048;
}

if ($state eq "NONE") {
  $cnty = "no";
  $city = "no";
} 
elsif ($state eq "Alabama") {
  $cnty = "sdf/data/Alabama/co01_d00.dat";
  $city = "sdf/data/Alabama/city";
} 
elsif ($state eq "Alaska") {
  $cnty = "sdf/data/Alaska/co02_d00.dat";
  $city = "sdf/data/Alaska/city";
} 
elsif ($state eq "Arizona") {
  $cnty = "sdf/data/Arizona/co04_d00.dat";
  $city = "sdf/data/Arizona/city";
}
elsif ($state eq "Arkansas") {
  $cnty = "sdf/data/Arkansas/co05_d00.dat";
  $city = "sdf/data/Arkansas/city";
} 
elsif ($state eq "California") {
  $cnty = "sdf/data/California/co06_d00.dat";
  $city = "sdf/data/California/city";
} 
elsif ($state eq "Colorado") {
  $cnty = "sdf/data/Colorado/co08_d00.dat";
  $city = "sdf/data/Colorado/city";
} 
elsif ($state eq "Connecticut") {
  $cnty = "sdf/data/Connecticut/co09_d00.dat";
  $city = "sdf/data/Connecticut/city";
} 
elsif ($state eq "Delaware") {
  $cnty = "sdf/data/Delaware/co10_d00.dat";
  $city = "sdf/data/Delaware/city";
} 
elsif ($state eq "District of Columbia") {
  $cnty = "sdf/data/DC/co11_d00.dat";
  $city = "sdf/data/DC/city";
} 
elsif ($state eq "Florida") {
  $cnty = "sdf/data/Florida/co12_d00.dat";
  $city = "sdf/data/Florida/city";
} 
elsif ($state eq "Georgia") {
  $cnty = "sdf/data/Georgia/co13_d00.dat";
  $city = "sdf/data/Georgia/city";
} 
elsif ($state eq "Hawaii") {
  $cnty = "sdf/data/Hawaii/co15_d00.dat";
  $city = "sdf/data/Hawaii/city";
} 
elsif ($state eq "Idaho") {
  $cnty = "sdf/data/Idaho/co16_d00.dat";
  $city = "sdf/data/Idaho/city";
} 
elsif ($state eq "Illinois") {
  $cnty = "sdf/data/Illinois/co17_d00.dat";
  $city = "sdf/data/Illinois/city";
}
elsif ($state eq "Indiana") {
  $cnty = "sdf/data/Indiana/co18_d00.dat";
  $city = "sdf/data/Indiana/city";
}
elsif ($state eq "Iowa") {
  $cnty = "sdf/data/Iowa/co19_d00.dat";
  $city = "sdf/data/Iowa/city";
}
elsif ($state eq "Kansas") {
  $cnty = "sdf/data/Kansas/co20_d00.dat";
  $city = "sdf/data/Kansas/city";
}
elsif ($state eq "Kentucky") {
  $cnty = "sdf/data/Kentucky/co21_d00.dat";
  $city = "sdf/data/Kentucky/city";
}
elsif ($state eq "Louisiana") {
  $cnty = "sdf/data/Louisiana/co22_d00.dat";
  $city = "sdf/data/Louisiana/city";
}
elsif ($state eq "Maine") {
  $cnty = "sdf/data/Maine/co23_d00.dat";
  $city = "sdf/data/Maine/city";
}
elsif ($state eq "Maryland") {
  $cnty = "sdf/data/Maryland/co24_d00.dat";
  $city = "sdf/data/Maryland/city";
}
elsif ($state eq "Massachusetts") {
  $cnty = "sdf/data/Massachusetts/co25_d00.dat";
  $city = "sdf/data/Massachusetts/city";
}
elsif ($state eq "Michigan") {
  $cnty = "sdf/data/Michigan/co26_d00.dat";
  $city = "sdf/data/Michigan/city";
}
elsif ($state eq "Minnesota") {
  $cnty = "sdf/data/Minnesota/co27_d00.dat";
  $city = "sdf/data/Minnesota/city";
}
elsif ($state eq "Mississippi") {
  $cnty = "sdf/data/Mississippi/co28_d00.dat";
  $city = "sdf/data/Mississippi/city";
}
elsif ($state eq "Missouri") {
  $cnty = "sdf/data/Missouri/co29_d00.dat";
  $city = "sdf/data/Missouri/city";
}
elsif ($state eq "Montana") {
  $cnty = "sdf/data/Montana/co30_d00.dat";
  $city = "sdf/data/Montana/city";
}
elsif ($state eq "Nebraska") {
  $cnty = "sdf/data/Nebraska/co31_d00.dat";
  $city = "sdf/data/Nebraska/city";
}
elsif ($state eq "Nevada") {
  $cnty = "sdf/data/Nevada/co32_d00.dat";
  $city = "sdf/data/Nevada/city";
}
elsif ($state eq "New Hampshire") {
  $cnty = "sdf/data/NewHampshire/co33_d00.dat";
  $city = "sdf/data/NewHampshire/city";
}
elsif ($state eq "New Jersey") {
  $cnty = "sdf/data/NewJersey/co34_d00.dat";
  $city = "sdf/data/NewJersey/city";
}
elsif ($state eq "New Mexico") {
  $cnty = "sdf/data/NewMexico/co35_d00.dat";
  $city = "sdf/data/NewMexico/city";
}
elsif ($state eq "New York") {
  $cnty = "sdf/data/NewYork/co36_d00.dat";
  $city = "sdf/data/NewYork/city";
}
elsif ($state eq "North Carolina") {
  $cnty = "sdf/data/NorthCarolina/co37_d00.dat";
  $city = "sdf/data/NorthCarolina/city";
}
elsif ($state eq "North Dakota") {
  $cnty = "sdf/data/NorthDakota/co38_d00.dat";
  $city = "sdf/data/NorthDakota/city";
}
elsif ($state eq "Ohio") {
  $cnty = "sdf/data/Ohio/co39_d00.dat";
  $city = "sdf/data/Ohio/city";
}
elsif ($state eq "Oklahoma") {
  $cnty = "sdf/data/Oklahoma/co40_d00.dat";
  $city = "sdf/data/Oklahoma/city";
}
elsif ($state eq "Oregon") {
  $cnty = "sdf/data/Oregon/co41_d00.dat";
  $city = "sdf/data/Oregon/city";
}
elsif ($state eq "Pennsylvania") {
  $cnty = "sdf/data/Pennsylvania/co42_d00.dat";
  $city = "sdf/data/Pennsylvania/city";
}
elsif ($state eq "Rhode Island") {
  $cnty = "sdf/data/RhodeIsland/co44_d00.dat";
  $city = "sdf/data/RhodeIsland/city";
}
elsif ($state eq "South Carolina") {
  $cnty = "sdf/data/SouthCarolina/co45_d00.dat";
  $city = "sdf/data/SouthCarolina/city";
}
elsif ($state eq "South Dakota") {
  $cnty = "sdf/data/SouthDakota/co46_d00.dat";
  $city = "sdf/data/SouthDakota/city";
}
elsif ($state eq "Tennessee") {
  $cnty = "sdf/data/Tennessee/co47_d00.dat";
  $city = "sdf/data/Tennessee/city";
}
elsif ($state eq "Texas") {
  $cnty = "sdf/data/Texas/co48_d00.dat";
  $city = "sdf/data/Texas/city";
}
elsif ($state eq "Utah") {
  $cnty = "sdf/data/Utah/co49_d00.dat";
  $city = "sdf/data/Utah/city";
}
elsif ($state eq "Vermont") {
  $cnty = "sdf/data/Vermont/co50_d00.dat";
  $city = "sdf/data/Vermont/city";
}
elsif ($state eq "Virginia") {
  $cnty = "sdf/data/Virginia/co51_d00.dat";
  $city = "sdf/data/Virginia/city";
}
elsif ($state eq "Washington") {
  $cnty = "sdf/data/Washington/co53_d00.dat";
  $city = "sdf/data/Washington/city";
}
elsif ($state eq "West Virginia") {
  $cnty = "sdf/data/WestVirginia/co54_d00.dat";
  $city = "sdf/data/WestVirginia/city";
}
elsif ($state eq "Wisconsin") {
  $cnty = "sdf/data/Wisconsin/co55_d00.dat";
  $city = "sdf/data/Wisconsin/city";
}
elsif ($state eq "Wyoming") {
  $cnty = "sdf/data/Wyoming/co56_d00.dat";
  $city = "sdf/data/Wyoming/city";
}
elsif ($state eq "American Samoa") {
  $cnty = "sdf/data/AmericanSamoa/co60_d00.dat";
  $city = "sdf/data/AmericanSamoa/city";
}
elsif ($state eq "Guam") {
  $cnty = "sdf/data/Guam/co66_d00.dat";
  $city = "sdf/data/Guam/city";
}
elsif ($state eq "N. Mariana Islands") {
  $cnty = "sdf/data/NMarianaIslands/co69_d00.dat";
  $city = "sdf/data/NMarianaIslands/city";
}
elsif ($state eq "Puerto Rico") {
  $cnty = "sdf/data/PuertoRico/co72_d00.dat";
  $city = "sdf/data/PuertoRico/city";
}
elsif ($state eq "Virgin Islands") {
  $cnty = "sdf/data/VirginIslands/co78_d00.dat";
  $city = "sdf/data/VirginIslands/city";
}

## Frequency
#
$frq =~ tr/0-9.//csd;

if (!$frq) {
  print "<b>ENTER A FREQUENCY!!!</b></html>";
  exit 1;
} 

if ($frq_val eq "GHz") {
  $frq_mhz = sprintf "%.4f", $frq * 1000;
  $frq_ghz = sprintf "%.4f", $frq;
} 
else {
  $frq_mhz = sprintf "%.4f", $frq;
  $frq_ghz = sprintf "%.4f", $frq / 1000;
} 

## Get Earth Dielectric Constant
#
$diecon =~ tr/0-9.//csd;

if (!$diecon || $diecon < 1 || $diecon > 81) {
  $diecon = 15;
}

## Get Earth Conductivity
#
$earcon =~ tr/0-9.//csd;

if (!$earcon || $earcon < 0 || $earcon > 10) {
  $earcon = 0.005;
}

## Climate
#
if ($climate eq "Equatorial") {
  $cli = 1;
}
elsif ($climate eq "Continental Subtropical") {
  $cli = 2;
}
elsif ($climate eq "Maritime Subtropical") {
  $cli = 3;
}
elsif ($climate eq "Desert") {
  $cli = 4;
}
elsif ($climate eq "Continental Temperate") {
  $cli = 5;
}
elsif ($climate eq "Maritime Temperate, Overland") {
  $cli = 6;
}
elsif ($climate eq "Maritime Temperate, Oversea") {
  $cli = 7;
}

## Get K Factor
#
if ($k eq "7/6") {
  $k = 7 / 6;
  $k_str = "7/6";
}
elsif ($k eq "4/3") {
  $k = 4 / 3;
  $k_str = "4/3";
}
elsif ($k eq "5/3") {
  $k = 5 / 3;
  $k_str = "5/3";
}
elsif ($k eq "7/4") {
  $k = 7 / 4;
  $k_str = "7/4";
} 
elsif ($k eq "2.0") {
  $k = 2.0;
  $k_str = "2.0";
}
elsif ($k eq "4.0") {
  $k = 4.0;
  $k_str = "4.0";
} 
elsif ($k eq "Infinity") {
  $k = 10000;
  $k_str = "Infinity";
}

$nunits = 179.3 * (ln ((1 / 0.04665) * (1 - (1 / $k))));

## Antenna Polarization
#
if ($polar eq "Vertical") {
  $ant = 1;
}
else {
  $ant = 0;
}

## Coverage Area
#
$coverage =~ tr/0-9.//csd;
   
if (!$coverage || $coverage <= 0 || $coverage >= 300) {
  $coverage = 20;
  $coverage_val = "miles";
}

if ($coverage_val eq "miles") {
  $cov_mi = sprintf "%.2f", $coverage;
  $cov_km = sprintf "%.2f", $coverage * 1.609344;
}
else {
  $cov_mi = sprintf "%.2f", $coverage / 1.609344;
  $cov_km = sprintf "%.2f", $coverage;
}

## Max dB Path Loss
#

$maxdb =~ tr/0-9.//csd;

if (!$maxdb || $maxdb > 230) {
  $maxdb = 230; 
}

## Situations
#
if ($sit eq "3 %") {
  $si = 0.03;
}
elsif ($sit eq "10 %") {
  $si = 0.10;
}
elsif ($sit eq "20 %") {
  $si = 0.20;
} 
elsif ($sit eq "30 %") {
  $si = 0.30;
}
elsif ($sit eq "40 %") {
  $si = 0.40;
} 
elsif ($sit eq "50 %") {
  $si = 0.50;
} 
elsif ($sit eq "60 %") {
  $si = 0.60;
}
elsif ($sit eq "70 %") {
  $si = 0.70;
}
elsif ($sit eq "80 %") {
  $si = 0.80;
}
elsif ($sit eq "90 %") {
  $si = 0.90;
}
elsif ($sit eq "97 %") {
  $si = 0.97;
}

if ($time eq "3 %") {
  $ti = 0.03;
} 
elsif ($time eq "10 %") {
  $ti = 0.10;
} 
elsif ($time eq "20 %") {
  $ti = 0.20;
} 
elsif ($time eq "30 %") {
  $ti = 0.30;
} 
elsif ($time eq "40 %") {
  $ti = 0.40;
} 
elsif ($time eq "50 %") {
  $ti = 0.50;
} 
elsif ($time eq "60 %") {
  $ti = 0.60;
} 
elsif ($time eq "70 %") {
  $ti = 0.70;
} 
elsif ($time eq "80 %") {
  $ti = 0.80;
} 
elsif ($time eq "90 %") {
  $ti = 0.90;
} 
elsif ($time eq "97 %") {
  $ti = 0.97;
} 

$diecon = sprintf "%.2f", $diecon;
$earcon = sprintf "%.3f", $earcon;
$nunits = sprintf "%.1f", $nunits;

# Draw me a web page
#
print <<EOF;
<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 3.2//EN">
<html>
<head>
<title>Longley-Rice Path Loss Analysis Results</title>
</head>
<body bgcolor="#D3D3D3" text="#000000" link="blue">
<center>
<h2>Longley-Rice Path Loss Analysis Results</h2>
</center>

<hr noshade>
<pre>
<u><b>Transmitter Site</b></u>

                          <b>Latitude : </b>$LAT1<b>&deg; (</b>$LAT1_D<b>&deg; </b>$LAT1_M<b>' </b>$LAT1_S<b>") North</b>
                         <b>Longitude : </b>$LON1<b>&deg; (</b>$LON1_D<b>&deg; </b>$LON1_M<b>' </b>$LON1_S<b>") West</b>
                    <b>Antenna Height : </b>$tx_ant_ht_m<b> meters (</b>$tx_ant_ht_ft<b> feet) AGL  Center-of-Radiation</b>
     <b>Highest Transmitted Frequency : </b>$frq_ghz <b>GHz  (</b>$frq_mhz <b>MHz)</b>
              <b>Antenna Polarization : </b>$polar
                   <b>Coverage Radius : </b>$cov_km<b> kilometers (</b>$cov_mi<b> miles)</b>
           <b>Maximum Path Loss Shown : </b>$maxdb<b> dB</b>

           <b>Receiver Antenna Height : </b>$rx_ant_ht_m<b> meters (</b>$rx_ant_ht_ft<b> feet) AGL  Center-of-Radiation</b>

<u><b>Longley-Rice Parameters</b></u>

         <b>Earth Dielectric Constant : </b>$diecon <b>(Relative Permittivity)</b>
              <b>Ground Conductivity  : </b>$earcon <b>Siemens/m</b>
                      <b>Climate Type : </b>$climate
   <b>Index of Atmospheric Refraction : </b>$nunits <b>N-units (parts per million)</b>
 <b>Effective Earth Radius (K Factor) : </b>$k_str</b>
<b>Confidence / Situation Variability : </b>$sit
    <b>Reliability / Time Variability : </b>$time
EOF
&flush(STDOUT);

open F, "> tx.qth";
  print F "Transmitter\n";
  print F "$LAT1\n";
  print F "$LON1\n";
  print F "$tx_ant_ht_ft\n";
close F;

open LR, "> tx.lrp";
  print LR "$diecon\n".
           "$earcon\n".
           "$nunits\n".
           "$frq_mhz\n".
           "$cli\n".
           "$ant\n".
           "$si\n".
           "$ti\n";
close LR; 

print "\n\n<b>Starting ... (will take a few minutes)\n";
&flush(STDOUT);

if ($city eq "no") {
    &System($args = "/usr/local/bin/splat -t tx.qth -L $rx_ant_ht_ft -R $cov_mi -db $maxdb -m $k -d sdf");
}
else {
    &System($args = "/usr/local/bin/splat -t tx.qth -L $rx_ant_ht_ft -s $city -b $cnty -R $cov_mi -db $maxdb -m $k -d sdf"); 
}

print "</b>\n";
&flush(STDOUT);

$file = "Transmitter-site_report.txt";

if (-f $file) {
  open F, "$file";
    while (<F>) {
      if (/^GND/) {
        ($a, $tx_elv_ft) = split ':';
      }
      if (/^HAA/) {
        ($a, $haat) = split ':';
      }
      if (/^  0/) {
        ($a, $A) = split ':';
      }
      if (/^ 45/) {
        ($a, $B) = split ':';
      } 
      if (/^ 90/) {
        ($a, $C) = split ':';
      }
      if (/^135/) {
        ($a, $D) = split ':';
      }
      if (/^180/) {
        ($a, $E) = split ':';
      } 
      if (/^225/) {
        ($a, $F) = split ':';
      } 
      if (/^270/) {
        ($a, $G) = split ':';
      }
      if (/^315/) {
        ($a, $H) = split ':';
      }
    }
  close F;

$p = "<b>";
$e = "</b>";

print "\n\n<u><b>Transmitter Site</b></u>\n\n";
printf "               %s Ground Elevation : %s%.1f%s meters (%s%.1f%s feet) AMSL %s\n",$p,$e,$tx_elv_ft * 0.3048,$p,$e,$tx_elv_ft,$p,$e;
&flush(STDOUT);
printf "   %s Height Above Average Terrain : %s%.1f%s meters (%s%.1f%s feet) %s\n",$p,$e,$haat * 0.3048,$p,$e,$haat,$p,$e;
&flush(STDOUT);
printf "%s Average Terrain at   0&deg; Azimuth : %s%.1f%s meters (%s%.1f%s feet) AMSL %s\n",$p,$e,$A * 0.3048,$p,$e,$A,$p,$e;
&flush(STDOUT);
printf "%s Average Terrain at  45&deg; Azimuth : %s%.1f%s meters (%s%.1f%s feet) AMSL %s\n",$p,$e,$B * 0.3048,$p,$e,$B,$p,$e;
&flush(STDOUT);
printf "%s Average Terrain at  90&deg; Azimuth : %s%.1f%s meters (%s%.1f%s feet) AMSL %s\n",$p,$e,$C * 0.3048,$p,$e,$C,$p,$e;
&flush(STDOUT);
printf "%s Average Terrain at 135&deg; Azimuth : %s%.1f%s meters (%s%.1f%s feet) AMSL %s\n",$p,$e,$D * 0.3048,$p,$e,$D,$p,$e;
&flush(STDOUT);
printf "%s Average Terrain at 180&deg; Azimuth : %s%.1f%s meters (%s%.1f%s feet) AMSL %s\n",$p,$e,$E * 0.3048,$p,$e,$E,$p,$e;
&flush(STDOUT);
printf "%s Average Terrain at 225&deg; Azimuth : %s%.1f%s meters (%s%.1f%s feet) AMSL %s\n",$p,$e,$F * 0.3048,$p,$e,$F,$p,$e;
&flush(STDOUT);
printf "%s Average Terrain at 270&deg; Azimuth : %s%.1f%s meters (%s%.1f%s feet) AMSL %s\n",$p,$e,$G * 0.3048,$p,$e,$G,$p,$e;
&flush(STDOUT);
printf "%s Average Terrain at 315&deg; Azimuth : %s%.1f%s meters (%s%.1f%s feet) AMSL %s\n",$p,$e,$H * 0.3048,$p,$e,$H,$p,$e;
&flush(STDOUT);
}

if ($quality eq "Low / Fast") {
  print "\n\n<b>Generating Low Quality JPEG Image ...</b>";
  &flush(STDOUT);

  &System($args = "/usr/bin/pnmtojpeg --quality=10 map.ppm > GBPPR_LR.jpg");

  print "\n\n<b>Map Coverage Colors - Estimated Path Loss in dB</b>\n\n";
  print "<table border=2 cellpadding=2><tr>\n";
  print "<td colspan=10 bgcolor=\"#FF0000\" align=center><b>80 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#FF8000\" align=center><b>90 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#FFA500\" align=center><b>100 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#FFCE00\" align=center><b>110 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#FFFF00\" align=center><b>120 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#B8FF00\" align=center><b>130 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#00FF00\" align=center><b>140 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#00D000\" align=center><b>150 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#00C4C4\" align=center><b>160 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#0094FF\" align=center><b>170 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#5050FF\" align=center><b>180 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#0026FF\" align=center><b>190 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#8E3FFF\" align=center><b>200 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#C436FF\" align=center><b>210 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#FF00FF\" align=center><b>220 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#FFC2CC\" align=center><b>230 dB</b></td></tr></table>\n";
  &flush(STDOUT);

  $size = (-s "GBPPR_LR.jpg");
  print "<p><b>Displaying $size byte JPEG Image ...</b></p>\n";
  print "<img src=\"tmp/$mon-$mday/$RAN/GBPPR_LR.jpg\">\n";
  print "</body>\n";
  print "</html>\n";
  &flush(STDOUT);
}
else {

  print "\n\n<b>Generating High Quality PNG Image ...</b>";
  &flush(STDOUT);
      
  &System($args = "/usr/bin/pnmtopng -compression 9 map.ppm > GBPPR_LR.png");

  print "\n\n<b>Map Coverage Colors - Estimated Path Loss in dB</b>\n\n";

  print "<table border=2 cellpadding=2><tr>\n";
  print "<td colspan=10 bgcolor=\"#FF0000\" align=center><b>80 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#FF8000\" align=center><b>90 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#FFA500\" align=center><b>100 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#FFCE00\" align=center><b>110 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#FFFF00\" align=center><b>120 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#B8FF00\" align=center><b>130 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#00FF00\" align=center><b>140 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#00D000\" align=center><b>150 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#00C4C4\" align=center><b>160 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#0094FF\" align=center><b>170 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#5050FF\" align=center><b>180 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#0026FF\" align=center><b>190 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#8E3FFF\" align=center><b>200 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#C436FF\" align=center><b>210 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#FF00FF\" align=center><b>220 dB</b></td>\n";
  print "<td colspan=10 bgcolor=\"#FFC2CC\" align=center><b>230 dB</b></td></tr></table>\n";
  &flush(STDOUT);

  $size = (-s "GBPPR_LR.png");
  print "<p><b>Displaying $size byte PNG Image ...</b></p>\n";
  print "<img src=\"tmp/$mon-$mday/$RAN/GBPPR_LR.png\">\n";
  print "</body>\n";
  print "</html>\n";
  &flush(STDOUT);
}

unlink $file, "map.ppm", "tx.qth", "tx.lrp", "sdf";

exit 0;
